package wvpaint;

/**
 *
 * @authors: Isaac Witmer and Daniel Vader
 * @acknowledgements: Cay S. Horstmann for some source code.
 * @TimeLog: 24 January 2009 3:30p-6:30p
 *           28 January 2009 4pm-6pm
 *           30 January 2009 9am-10am
 *           30 January 2009 12pm-1pm
 *           1  February 2009 2pm-5pm
 *           1  February 2009 7pm-8pm
 * 
 */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.awt.geom.*;
import javax.swing.*;
import java.io.*;

/**
 * A Simple paint program that can create squares, circles, text, and simple
 * lines on a canvas.  The squares or the circles can be frames or filled shapes
 * as defined by user on creation.  The user can change the color of any
 * indivudual object already on the canvas, or set the color of the object
 * they are about to create.  The user can move these shapes around, remove
 * them from the canvas, or move them into the foreground or the background.
 * A seperate frame with a help button will open the user's internet browser to
 * a Help File webpage with instructions on how to use the program.
 */
public class WVPaint {

    public static void main(String[] args) {
        EventQueue.invokeLater(new Runnable() {

            public void run() {
                PaintFrame frame = new PaintFrame();
                frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
                frame.setVisible(true);

                Help helpU = new Help();
                helpU.setVisible(true);

            }
        });
    }
}

/**
 * A frame containing a hyperlink to Help information for the user
 */
class Help extends JFrame {

    private ActionHelp hListener;

    public Help() {

        setSize(100, 100);//set the frame size
        this.setLocation(PaintFrame.DEFAULT_WIDTH, 0); //set frame location
        //according to the width of the main frame.

        hListener = new ActionHelp();//initialize actionlistener class

        //Create the hyperlink button and corresponding JPanel
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(1, 1));
        JButton helpU = new JButton("HELP ME!");
        helpU.addActionListener(hListener);
        panel.add(helpU);
        getContentPane().add(panel, BorderLayout.CENTER);

    }

    /**
     * When the user clicks the help button they will be sent to our help
     * website.
     */
    private class ActionHelp implements ActionListener {

        public void actionPerformed(ActionEvent event) {
            try {
                Process pc = Runtime.getRuntime().exec("cmd.exe /c start http://www.goshen.edu/~danieltv/wvpaint");
            } catch (IOException ex) {
                System.out.println(ex.getMessage());
                System.out.println();
            }

        }
    }
}

/**
 * A frame containing a panel and buttons
 */
class PaintFrame extends JFrame {

    // So we can call methods in the PaintComponent class
    private PaintComponent component;
    //List of J Items that require acion/item listeners    
    private JCheckBox fill;
    private JComboBox shapeCB,  colorCB;
    private JButton toFront,  toBack;
    private JButton goColor,  goText;
    private JTextArea textBox;
    private JPanel panel;
    private JMenuBar menu;
    //Variables that Paint Frame needs to change
    private String uText;
    private String curShape;
    private Color curColor;
    //The size of the Paint Program
    public static final int DEFAULT_WIDTH = 600;
    public static final int DEFAULT_HEIGHT = 400;

    /**
     * The Frame for the program
     * It will hold the paint component
     */
    public PaintFrame() {
        setTitle("MouseTest");
        setSize(DEFAULT_WIDTH, DEFAULT_HEIGHT);

        //setup private components
        build();

        // add component to frame
        component = new PaintComponent();
        panel = new JPanel();
        panel.setLayout(new GridLayout(2, 5));
        panel.add(shapeCB);
        panel.add(textBox);
        panel.add(colorCB);
        panel.add(toFront);
        panel.add(fill);
        panel.add(goText);
        panel.add(goColor);
        panel.add(toBack);
        panel.setBorder(BorderFactory.createEtchedBorder());
        getContentPane().add(panel, BorderLayout.NORTH);
        add(component);

    }

    /**
     * A method that constructs all the objects to be added to the panel
     * It should be called once, in the beginning.
     */
    private void build() {

        PaintAction listener = new PaintAction();

        menu = new JMenuBar();

        fill = new JCheckBox("Filled Shape");

        goText = new JButton("Save Text");
        goText.addActionListener(listener);
        goText.setEnabled(false);

        goColor = new JButton("Apply to Selected");
        goColor.addActionListener(listener);

        shapeCB = new JComboBox();
        shapeCB.addItem("Oval");
        shapeCB.addItem("Square");
        shapeCB.addItem("Line");
        shapeCB.addItem("Text");
        shapeCB.addItemListener(listener);
        curShape = (String) shapeCB.getSelectedItem();

        toFront = new JButton("Selected to Front");
        toFront.addActionListener(listener);

        toBack = new JButton("Selected to Back");
        toBack.addActionListener(listener);


        textBox = new JTextArea();
        textBox.setText("Type Here");
        uText = textBox.getText();
        textBox.setEditable(true);
        textBox.setEnabled(false);
        textBox.setBorder(BorderFactory.createEtchedBorder());

        colorCB = new JComboBox();
        colorCB.addItem("BLACK");
        colorCB.addItem("BLUE");
        colorCB.addItem("CYAN");
        colorCB.addItem("GRAY");
        colorCB.addItem("MAGENTA");
        colorCB.addItem("GREEN");
        colorCB.addItem("ORANGE");
        colorCB.addItem("PINK");
        colorCB.addItem("RED");
        colorCB.addItem("WHITE");
        colorCB.addItem("YELLOW");
        colorCB.addItemListener(listener);
        curColor = Color.BLACK;
    }

    /**
     * The Paint Action class
     * This controls the actions that occur
     */
    private class PaintAction implements ActionListener, ItemListener {

        /**
         * Responds if an action is preformed, ie: button
         * @param event the event that is passed in when the action is performed
         */
        public void actionPerformed(ActionEvent event) {
            Object action = event.getSource();
            if (action.equals(toFront)) {
                //move item to he front of the array
                component.moveToFront();

            } else if (action.equals(toBack)) {
                //move item to the back of the array
                component.moveToBack();
            } else if (action.equals(goText)) {
                uText = textBox.getText();
            } else {
                component.paintColor();
            }
        }

        /**
         * Responds if an item is changed, ie: Combobox or checkbox
         * @param event the event that is passed in when the item is changed
         */
        public void itemStateChanged(ItemEvent event) {
            Object action = event.getSource();
            if (action.equals(shapeCB)) {
                curShape = (String) shapeCB.getSelectedItem();
                System.out.println(curShape);
                if (curShape.equals("Text")) {
                    goText.setEnabled(true);
                    textBox.setEnabled(true);
                } else {
                    goText.setEnabled(false);
                    textBox.setEnabled(false);
                }
            } else if (action.equals(colorCB)) {

                String colorStr = (String) colorCB.getSelectedItem();
                System.out.println(colorStr);

                if (colorStr.equals("BLACK")) {
                    curColor = Color.BLACK;
                } else if (colorStr.equals("BLUE")) {
                    curColor = Color.BLUE;
                } else if (colorStr.equals("CYAN")) {
                    curColor = Color.CYAN;
                } else if (colorStr.equals("GRAY")) {
                    curColor = Color.GRAY;
                } else if (colorStr.equals("GREEN")) {
                    curColor = Color.GREEN;
                } else if (colorStr.equals("MAGENTA")) {
                    curColor = Color.MAGENTA;
                } else if (colorStr.equals("ORANGE")) {
                    curColor = Color.ORANGE;
                } else if (colorStr.equals("PINK")) {
                    curColor = Color.PINK;
                } else if (colorStr.equals("RED")) {
                    curColor = Color.RED;
                } else if (colorStr.equals("WHITE")) {
                    curColor = Color.WHITE;
                } else {
                    curColor = Color.YELLOW;
                }
            }
        }
    }

    /**
     * A component with mouse operations for adding and removing 
     * rectangles, ovals, lines or text
     */
    private class PaintComponent extends JComponent {

        private static final int SIDELENGTH = 20;
        private ArrayList<PaintObject> doodle;
        private PaintObject current; //current selected shape

        /**
         * the contstructor
         */
        public PaintComponent() {

            doodle = new ArrayList<PaintObject>();
            current = null;

            addMouseListener(new MouseHandler());
            addMouseMotionListener(new MouseMotionHandler());
        }

        /**
         * The method that gets called when the UI demands/requests an update
         * @param g the Graphics object
         */
        public void paintComponent(Graphics g) {

            Graphics2D g2 = (Graphics2D) g;

            // Draws all the objects each time
            for (PaintObject r : doodle) {
                r.draw(g2);
            }
        }

        /**
         * Finds the first "doodle" containing a point.
         * @param p a point
         * @return the first square that contains p
         */
        public PaintObject find(Point2D p) {
            for (PaintObject r : doodle) {
                if ((r.getObject()).contains(p)) {
                    return r;
                }
            }
            return null;
        }

        /**
         * Adds a square to the collection.
         * @param p the center of the square
         */
        public void add(Point2D p) {
            double x = p.getX();
            double y = p.getY();

            if (curShape.equals("Oval")) {
                current = new OurOval(x, y, curColor, fill.isSelected());
            } else if (curShape.equals("Square")) {
                current = new OurRect(x, y, curColor, fill.isSelected());
            } else if (curShape.equals("Line")) {
                current = new OurLine(x, y, curColor);
            } else if (curShape.equals("Text")) {
                current = new OurText(uText, x, y, curColor);
            }
            doodle.add(current);
            repaint();
        }

        /**
         * Removes a square from the collection.
         * @param s the frame around the object to remove
         */
        private void remove(Rectangle2D s) {
            if (s == null) {
                return;
            }
            if (s == current) {
                current = null;
            }
            doodle.remove(s);
            repaint();
        }

        /**
         * Changes the color of the selected shape
         * to the color defined by the ComboBox
         */
        private void paintColor() {
            if (current == null) {
            } else {
                current.setColor(curColor);
                repaint();
            }
        }

        /**
         * Moves the current shape (in the Paint Component)
         * to the BACK of the doodle array
         */
        private void moveToBack() {
            if (current == null) {
            } else {
                int end = 0;
                int start = doodle.indexOf(current);
                for (int i = start; i > end; i--) { //start at the index, and go down until we get to the one before the first

                    doodle.set(i, doodle.get(i - 1));
                }
                doodle.set(0, current);
                repaint();
            }
        }

        /**
         * Moves the current shape (in the Paint Component)
         * to the FRONT of the doodle array
         */
        private void moveToFront() {
            if (current == null) {
            } else {
                int end = doodle.size() - 1;
                int start = doodle.indexOf(current);
                for (int i = start; i < end; i++) { //start at the index, and go up until we get to the one before the last

                    doodle.set(i, doodle.get(i + 1));
                }
                doodle.set(end, current);
                repaint();
            }
        }

        /**
         * A class that handles the Mouse
         */
        private class MouseHandler extends MouseAdapter {

            /**
             * Adds a new PaintObject when the mouse is pressed
             * if a object is not selected
             * @param event the mouse event
             */
            public void mousePressed(MouseEvent event) {
                // add a new square if the cursor isn't inside a square
                current = find(event.getPoint());
                if (current == null) {
                    add(event.getPoint());
                }
                moveToFront();
            }

            /**
             * Checks if the object has been double clicked
             * and Removes it
             * @param event the mouse event
             */
            public void mouseClicked(MouseEvent event) {
                current = find(event.getPoint());
                if (current != null && event.getClickCount() >= 2) {
                    doodle.remove(current);
                    repaint();
                }
            }
        }

        /**
         * A class that handles the Mouse Motion
         */
        private class MouseMotionHandler implements MouseMotionListener {

            /**
             * Changes the cursor of the mouse
             * if the cursor is overtop of a PaintObject
             * @param event the mouse event
             */
            public void mouseMoved(MouseEvent event) {
                if (find(event.getPoint()) == null) {
                    setCursor(Cursor.getDefaultCursor());
                } else {
                    setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
                }
            }

            /**
             * Moves the object if the object is selected
             * @param event the mouse event
             */
            public void mouseDragged(MouseEvent event) {
                if (current != null) {
                    int x = event.getX();
                    int y = event.getY();

                    // drag the current rectangle to center it at (x, y)
                    current.move(x, y);
                    repaint();
                }
            }
        }
    }
}
